@@ -3,34 +3,18 @@
 # Scan for camera
 # Save mkv files
 ##########################################################################
-# 7-9% CPU usage
+# Results
+# 1080p 30fps capture, 30fps save, 30fps display
+# 4% CPU usage
 ##########################################################################
-import cv2
-import logging
-import time
-import numpy as np
-from datetime import datetime
-from queue import Queue
-import platform
 
-# Probe the cameras, return indeces, fourcc, default resolution
-def probeCameras():
-    # check for up to 10 cameras
-    index = 0
-    arr = []
-    i = 10
-    while i > 0:
-        cap = cv2.VideoCapture(index)
-        if cap.read()[0]:
-            tmp = cap.get(cv2.CAP_PROP_FOURCC)
-            fourcc = "".join([chr((int(tmp) >> 8 * i) & 0xFF) for i in range(4)])
-            width = cap.get(cv2.CAP_PROP_FRAME_WIDTH)
-            height = cap.get(cv2.CAP_PROP_FRAME_HEIGHT)
-            cap.release()
-            arr.append({"index": index, "fourcc": fourcc, "width": width, "height": height})
-        index += 1
-        i -= 1
-    return arr
+# OpenCV
+import cv2
+
+# System
+import logging, time, platform
+from datetime import datetime
+from camera.utils import probeCameras
 
 # Camera Signature
 # In case we have multiple cameras, we can search for default driver settings
@@ -46,7 +30,7 @@ fourccSig = "\x16\x00\x00\x00"
 camera_index = 0
 
 # Scan all camera
-camprops = probeCameras()
+camprops = probeCameras(10)
 # Try to find the one that matches our signature
 score = 0
 for i in range(len(camprops)):
@@ -61,25 +45,30 @@ for i in range(len(camprops)):
         score = tmp
         camera_index = i
 
-# Camera configuration file
-# -Dell Inspiron 15 internal camer
-# from configs.dell_internal_configs  import configs as configs
-# -Eluktronics Max-15 internal camera
-from configs.eluk_configs import configs as configs
-# -Generic webcam
-#from configs.generic_1080p import configs as configs
-# -Nano Jetson IMX219 camera
-# from configs.nano_IMX219_configs  import configs as configs
-# -Raspberry Pi v1 & v2 camera
-# from configs.raspi_v1module_configs  import configs as configs
-# from configs.raspi_v2module_configs  import configs as configs
-# -ELP MAX15 internal camera
-# from configs.ELP1080p_configs  import configs as configs
-# -FLIR Lepton 3.5
-# from configs.FLIRlepton35 import confgis as configs
+configs = {
+    'camera_res'      : (1280, 720 ),   # width & height
+    'exposure'        : -6,             # -1,0 = auto, 1...max=frame interval, 
+    'autoexposure'    : 1.0,            # depends on camera: 0.25 or 0.75(auto), -1,0,1
+    'fps'             : 30,             # 15, 30, 40, 90, 120, 180
+    'fourcc'          : -1,             # n.a.
+    'buffersize'      : -1,             # n.a.
+    'output_res'      : (-1, -1),       # Output resolution, -1,-1 no change
+    'flip'            : 0,              # 0=norotation 
+                                        # 1=ccw90deg 
+                                        # 2=rotation180 
+                                        # 3=cw90 
+                                        # 4=horizontal 
+                                        # 5=upright diagonal flip 
+                                        # 6=vertical 
+                                        # 7=uperleft diagonal flip
+    'displayfps'       : 30             # frame rate for display server
+    }
 
-# Display
-display_interval = 1.0/configs['displayfps']
+if configs['displayfps'] >= configs['fps']:
+    display_interval = 0
+else:
+    display_interval = 1.0/configs['displayfps']
+    
 window_name      = 'Camera'
 font             = cv2.FONT_HERSHEY_SIMPLEX
 textLocation0    = (10,20)
@@ -94,26 +83,19 @@ cv2.namedWindow(window_name, cv2.WINDOW_AUTOSIZE) # or WINDOW_NORMAL
 logging.basicConfig(level=logging.INFO) # options are: DEBUG, INFO, ERROR, WARNING
 logger = logging.getLogger("Main")
 
-# Setting up input and output Queue
-captureQueue = Queue(maxsize=64)
-storageQueue = Queue(maxsize=64)
-# here add additional queues if you have more than one camera. 
-# Each camera needs capture and saving queue
-
 # Setting up Storage
 from camera.streamer.mkvstorageserver import mkvServer
-print("Starting Storage Server")
+logger.log(logging.INFO, "Starting Storage Server")
 now = datetime.now()
 fps  = configs['fps']
 size = configs['camera_res']
 filename = now.strftime("%Y%m%d%H%M%S") + ".mkv"
 mkv = mkvServer("C:\\temp\\" + filename, fps, size)
-print("Starting Storage Server")
-mkv.start(storageQueue)
+logger.log(logging.INFO, "Starting Storage Server")
+mkv.start()
 
 # Create camera interface
-print("Starting Capture")
-
+logger.log(logging.INFO, "Starting Capture")
 # Create camera interface based on computer OS you are running
 # plat can be Windows, Linux, MaxOS
 plat = platform.system()
@@ -127,36 +109,38 @@ if plat == 'Linux':
 else:
     from camera.capture.cv2capture import cv2Capture
     camera = cv2Capture(configs, camera_index)
-print("Getting Images")
-camera.start(captureQueue)
-
-print("Getting Images")
-camera.start(captureQueue)
+logger.log(logging.INFO, "Getting Images")
+camera.start()
 
 # Initialize Variables
-num_frames_sent      = 0           # keep track of data cubes sent to storage
-last_xps_time        = time.time() # keep track of time to dispay performance
-last_display         = time.time() # keeo track of time to display images
-num_frames_displayed = 0           # keep trakc of how many frames are displayed
-measured_dps         = 0           # computed in main thread, number of frames displayed per second
+num_frames_sent      = 0                   # keep track of data cubes sent to storage
+last_time            = time.perf_counter() # keep track of time to dispay performance
+last_display         = time.perf_counter() # keeo track of time to display images
+num_frames_displayed = 0                   # keep trakc of how many frames are displayed
+measured_dps         = 0                   # computed in main thread, number of frames displayed per second
 
 # Main Loop
-while(True):
-    current_time = time.time()
+stop=False
+while(not stop):
+    current_time = time.perf_counter()
 
     # wait for new image
-    (frame_time, frame) = captureQueue.get(block=True, timeout=None)
-    # if you have two cameras with different fps settings, we need to figure out here how to 
-    # obtain images in non blocking fashion as the slowest would prevail and buffer over runs on faster
-    
-    if not storageQueue.full():
-        storageQueue.put((frame_time, frame), block=False) 
+    (frame_time, frame) = camera.capture.get(block=True, timeout=None)
+    while not camera.log.empty():
+        (level, msg) = camera.log.get_nowait()
+        logger.log(level, "{}".format(msg))
+
+    if not mkv.queue.full():
+        mkv.queue.put_nowait((frame_time, frame)) 
         num_frames_sent += 1
     else:
         logger.log(logging.WARNING, "Status:Storage Queue is full!")
+    while not mkv.log.empty():
+        (level, msg)=mkv.log.get_nowait()
+        logger.log(level, "{}".format(msg))
 
-    # Display performance in main loop
-    if current_time - last_xps_time >= 5.0:
+    # Performances in main loop
+    if (current_time - last_time) >= 5.0:
         # how many frames did we send to storage
         measured_fps_sent = num_frames_sent/5.0
         logger.log(logging.INFO, "Status:frames sent to storage per second:{}".format(measured_fps_sent))
@@ -165,9 +149,10 @@ while(True):
         measured_dps = num_frames_displayed/5.0
         logger.log(logging.INFO, "Status:frames displayed per second:{}".format(measured_dps))
         num_frames_displayed = 0
-        last_xps_time = current_time
+        last_time = current_time
 
-    if (current_time - last_display) >= display_interval:
+    # if you want to run at full speed remove the if statement
+    if (current_time - last_display) >= 0.8*display_interval:
         frame_display = frame.copy()
         # This section creates significant delay and we need to throttle the display to maintain max capture and storage rate
         cv2.putText(frame_display,"Capture FPS:{} [Hz]".format(camera.measured_fps), textLocation0, font, fontScale, fontColor, lineType)
@@ -175,8 +160,7 @@ while(True):
         cv2.putText(frame_display,"Storage FPS:{} [Hz]".format(mkv.measured_cps),    textLocation2, font, fontScale, fontColor, lineType)
         cv2.imshow(window_name, frame_display)
         # quit the program if users enter q or closes the display window
-        if cv2.waitKey(1) & 0xFF == ord('q'): # this likely is the reason that display frame rate is not faster than 60fps.
-            break
+        if cv2.waitKey(1) & 0xFF == ord('q'): stop = True
         last_display = current_time
         num_frames_displayed += 1
 
